from ..imports import *
from .graph_utils import *
from .utils import *
from collections import defaultdict
# react_analyzer/utils/invert_utils.py
def invert_to_symbol_map(graph: dict, include_kinds: Optional[Set[str]] = None) -> dict:
    include_kinds = set(include_kinds or [])

    # Build export-kind index per module
    module_kinds: Dict[str, Dict[str, str]] = {
        mod: (data.get('kinds') or {}) for mod, data in graph.get('nodes', {}).items()
    }

    out: Dict[str, Dict[str, Set[str]]] = {}

    # Export side
    for mod, data in graph.get('nodes', {}).items():
        kinds = data.get('kinds', {})  # {name: kind}
        for name, k in kinds.items():
            if include_kinds and k not in include_kinds:
                continue
            out.setdefault(name, {'exported_in': set(), 'imported_in': set()})
            out[name]['exported_in'].add(mod)

    # Import side: keep only names whose DEST module exports them with a matching kind
    for e in graph.get('edges', []):
        dst = e.get('to')  # module path we import from
        dst_kinds = module_kinds.get(dst, {})
        for n in (e.get('named') or []):
            if n in ('*', '<side-effect>', '<default>'):
                continue
            if include_kinds:
                k = dst_kinds.get(n)
                if k not in include_kinds:
                    continue
            out.setdefault(n, {'exported_in': set(), 'imported_in': set()})
            out[n]['imported_in'].add(e['from'])

    # to lists
    return {
        k: {'exported_in': sorted(v['exported_in']),
            'imported_in': sorted(v['imported_in'])}
        for k, v in out.items()
    }

def invert_to_function_map(graph: dict) -> dict:
    # WARNING: this will also pull non-callable consts!
    return invert_to_symbol_map(graph, include_kinds={'function','const','let','var'})


def invert_to_variable_map(graph: dict) -> dict[str, dict[str, list[str]]]:
    """
    Build: {
      varName: {"exported_in": [files...], "imported_in": [files...] }
    }
    """
    out = defaultdict(lambda: {"exported_in": [], "imported_in": []})

    # where variables are defined/exported
    for fp, node in (graph.get("nodes") or {}).items():
        for name, kind in (node.get("kinds") or {}).items():
            if kind in {"const", "let", "var"}:
                out[name]["exported_in"].append(fp)

    # where they are imported (follow edges' named imports)
    for e in (graph.get("edges") or []):
        for nm in (e.get("named") or []):
            if nm in {"<default>", "<side-effect>", "<reexport>"}:
                continue
            # we don't *know* it's a variable here, but if it *is* one it'll be present above
            out[nm]["imported_in"].append(e["from"])

    # de-dup + sort
    for v in out.values():
        v["exported_in"] = sorted(set(v["exported_in"]))
        v["imported_in"] = sorted(set(v["imported_in"]))

    return dict(out)



from abstract_utilities import *

_BASE_DIR = get_caller_dir()

def safe_split(string, char, i=None, default=False):
    """Safely split a string by a character and optionally return index i."""
    if string is None or char is None:
        return string

    string_str, char_str = str(string), str(char)
    if char_str not in string_str:
        return string

    parts = string_str.split(char_str)

    if i is None:
        return parts

    if is_number(i):
        i = int(i)
        if i < len(parts):
            return parts[i]
        if default:
            return string if default is True else default
        return
    return string if default is True else default


def safe_slice(obj, i=None, k=None, default=False):
    """Safely slice an object like a list or string with defaults."""
    if obj is None or isinstance(obj, bool):
        return obj if default is True else default if default else None

    obj_len = len(obj)

    # If no indices provided, return as is (or default)
    if i is None and k is None:
        return obj if default is True else default if default else None

    # Normalize negative indices
    if isinstance(i, int) and i < 0:
        i = obj_len + i
    if isinstance(k, int) and k < 0:
        k = obj_len + k

    # Bound indices
    if i is not None:
        i = max(0, min(i, obj_len))
    if k is not None:
        k = max(0, min(k, obj_len))

    try:
        return obj[i:k]
    except Exception:
        return obj if default is True else default if default else None
def safe_join(*paths):
    paths = list(paths)
    paths = [path for path in paths if path]
    return os.path.join(*paths)
def get_inside(text,char):
    parts = text.split(char)

    part = safe_slice(parts,1,-1) or []
    return char.join(part)
def get_insides(text,char):
    parts = safe_split(text,char)
    returns = []
    for i,part in enumerate(parts):
        if i !=0 and i%2 != float(0):
            returns.append(part)
    return returns

def get_content_lines(contents=None,filepath=None):
    if not contents and filepath and os.path.isfile(filepath):
        contents = read_from_file(filepath)
    if contents:
        line_spl=None
        if isinstance(contents,str): 
            line_spl = contents.split('\n')
        if isinstance(contents,list):
            line_spl = contents
        return line_spl
def get_index_content(contents=None,filepath=None):
    line_spl = get_content_lines(contents=contents,filepath=filepath)
    export_ls=[]
    exp_all = False
    filename='.'
    if filepath:
        dirname = os.path.dirname(file_path)
        basename = os.path.basename(file_path)
        filename,ext = os.path.splitext(basename)
    for line in line_spl:
        if line.startswith('export'):
            item = eatAll(line.split('(')[0].split('=')[0].split('function')[-1].split('const')[-1].split(' ')[-1],[' ','','\n','\t',';'])
            if 'default ' in line:
                export_ls.append("export {"+f"default as {item}"+"}"+f" from './{filename}';")
            elif exp_all == False:
                exp_all = True
                export_ls.append(f"export * from './{filename}';")
    index_cont = '\n'.join(export_ls)
    return index_cont
def create_script_dir(contents=None,file_path=None,script_dir=None):
    line_spl = get_content_lines(contents=contents,file_path=file_path)
    content_lines = line_spl[1:]
    contents = '\n'.join(content_lines)
    index_cont = get_index_content(contents=content_lines)
    if script_dir:
        dirname = os.path.dirname(file_path)
        basename = os.path.basename(file_path)
        filename,ext = os.path.splitext(basename)
        if script_dir == True:
            script_dir = os.path.join(dirname,filename)
        if not os.path.isdir(script_dir):
            create_dirs(script_dir)
        os.path.join(script_dir,basename)
    index_path = os.path.join(dirbase,'index.ts')
    base_path = os.path.join(dirbase,basename)
    write_to_file(contents=contents,file_path=base_path)
    write_to_file(contents=index_cont,file_path=index_path)

def raw_create_dirs(*paths):
    full_path = safe_join(*paths)
    partial_paths = full_path.split('/')
    paths = [path for path in partial_paths if path]
    for i,path in enumerate(paths):
        if i == 0:
            full_path = path
        else:
            full_path = safe_join(full_path,path)
        os.makedirs(full_path,exist_ok=True)
    return full_path
def create_dirs(directory,child=None):
    full_path = safe_join(directory,child)
    if not os.path.exists(full_path):
        full_path = raw_create_dirs(full_path)
    return full_path
def get_base_dir(directory = None):
    return directory or _BASE_DIR
def create_base_path(directory=None,child=None):
    directory = get_base_dir(directory = directory)
    return safe_join(directory,child)
def create_base_dir(directory=None,child=None):
    full_path = create_base_path(directory=directory,child=child)
    if not os.path.exists(full_path):
        base_path = create_dirs(full_path)
    return base_path




from ..titles.titles import pad_or_trim
from ..titles.title_variants import title_variants_from_domain
from ..generators.metadata_builder import generate_metadata

### Imports from user-provided files
##from .social_media import create_post, create_post_url  # :contentReference[oaicite:4]{index=4}
##from .social_media_utils import get_output_params       # :contentReference[oaicite:5]{index=5}
##from .twitter import prepare_tweet                      # :contentReference[oaicite:6]{index=6}
##from .metadata import generate_meta_tags                # :contentReference[oaicite:7]{index=7}


def social_share(platform, *, text, url, via=None, hashtags=None):
    """
    Create a platform-optimized text-only social share post.
    Example:
        social_share("x", text="My post!", url="https://mysite.com/123")
    """
    output = create_post(
        platform,
        text=text,
        url=url,
        via=via,
        hashtags=hashtags
    )
    return output


def social_share_url(platform, *, text, url, via=None, hashtags=None):
    """
    Create a share link:
        https://twitter.com/intent/tweet?text=...
    """
    share_url = create_post_url(
        platform,
        text=text,
        url=url,
        via=via,
        hashtags=hashtags
    )
    return share_url


def generate_full_social_package(info):
    """
    Produces:
      - SEO metadata
      - OpenGraph tags
      - Twitter card
      - Thread/FB/X/Email share links
      - Hashtags
      - Clean padded titles/descriptions

    This is the FINAL unified API.
    """
    # STEP 1 → SEO metadata
    meta = generate_metadata(info)

    url = meta.get("canonical")
    title = meta.get("title")
    desc = meta.get("description")
    keywords = meta.get("keywords", "")

    platforms = ["x", "facebook", "threads", "mailto", "minds"]

    share_links = {
        platform: social_share_url(
            platform,
            text=desc,
            url=url,
            via=info.get("via"),
            hashtags=keywords
        )
        for platform in platforms
    }

    share_posts = {
        platform: social_share(
            platform,
            text=desc,
            url=url,
            via=info.get("via"),
            hashtags=keywords
        )
        for platform in platforms
    }

    tweet = prepare_tweet(
        keyWords=keywords,
        pageTitle=title,
        description=desc,
        title=title,
        link_address=url
    )

    return {
        "metadata": meta,
        "open_graph_tags": generate_meta_tags(meta, json_path=info.get("json_path", "")),
        "tweet": tweet,
        "share_links": share_links,
        "share_posts": share_posts,
        "title_variants": title_variants_from_domain(info.get("domain", "")),
    }



from abstract_utilities import *
from abstract_react import *
def split_caps(string):
    alphs_lower = 'abcdefghijklmnopqrstuvwxyz'
    alphs_upper = alphs_lower.upper()
    spl_chars = ''
    for char in string:
        if char in alphs_upper:
            char=f"_{char.lower()}"
        spl_chars+=char
    return spl_chars
def make_all_aliases(kinds,aliases):
    return_aliases = []
    
    for alias in aliases:
        return_aliases.append(alias)
        if '_' not in alias:
            alias = split_caps(alias)
        alias_spl = [al.lower() for al in alias.split('_') if al]
        alias_lower = ''.join(alias_spl)
        return_aliases.append(alias_lower)
        for i,alias_ap in enumerate(alias_spl):
            if i != 0:  
                alias_ap = capitalize(alias_ap)
            alias_spl[i] = alias_ap
        alias_upper = ''.join(alias_spl)
        return_aliases.append(alias_upper)
    return list(set([alias for alias in return_aliases if alias not in kinds]))
def are_equal(*nums):
    prev_num = None
    for num in nums:
        if prev_num == None:
            prev_num = num
        elif prev_num != num:
            return False
    return True
def get_full_args(string,fname):
    string_parts = {"fname":fname,"args":None,"returns":None}
    chars_js = {"(":1,")":0}
    string_spl = string.split('(')
    string_parts["fname"] = string_spl[0].split(' ')[-1]
    string_part = '('.join(string_spl[1:])
    for i,char in enumerate(string_part):
        if char in chars_js:
            chars_js[char]+=1
        values = chars_js.values()
        if are_equal(*values):
           
           string_parts["args"] = eatAll(string_part[:i],[')',' ','\n','\t','('])
           returns = string_part[i:].split('{')[0]
           
           string_parts["returns"] = eatAll(returns,['(',')',':',' ','\t','\n']) if [ret for ret in returns.split(' ') if ret] else None
           return string_parts
def create_alias_funcs(string,fname,aliases):
    full_args = get_full_args(string,fname)
    for i,alias in enumerate(aliases):
        args = full_args.get('args')
        input_args = ','.join([arg.split(':')[0] for arg in args.split(',') if ':' in arg])
        returns = full_args.get('returns')
        
        returns = f":{returns}" if returns else ""
        aliases[i] = f"\nexport function {alias}(\n\t{args}\n\t){returns} {{\n\t\treturn {fname}({input_args});\n\t}}\n"
    return aliases
def roll_types(kinds):
    types={}
    for key,value in kinds.items():
        if value not in types:
            types[value] = []
        if key not in types[value]:
            types[value].append(key)
    return types
def update_list_value(*lists):
    nu_list = []
    for li in lists:
        nu_list+=li
    return list(set(li))
    
def fnames_in_kinds(all_declared,file):
    contents_js = all_declared["scripts"][file]
    kinds = contents_js.get('kinds')
    fnames = all_declared.get('fnames')
    for fname,fname_values in fnames.items():
        variables = list(kinds.keys())
        if fname_values.get('file') == None and fname in variables:
            all_declared["fnames"][fname]['file'] = file
            all_declared["fnames"][fname]['aliases'] = make_all_aliases(kinds,fname_values.get('aliases'))
            contents_js["fnames"].append(fname)
    all_declared["scripts"][file] = contents_js   
    return all_declared
def get_all_func_names(json_path,all_declared=None):
    all_declared= all_declared or {"fnames":{},"scripts":{},"kinds":{},"types":{}}
    data = safe_read_from_json(json_path)
    dirname = os.path.dirname(json_path)
    dirs,sub_files = get_files_and_dirs(directory,allowed_exts='.ts')
    declared_fnames = all_declared.get('fnames')
    fnames = list(data.keys())
    for fname in fnames:
        if fname not in all_declared['fnames']:
            vals = make_list(data.get(fname) or fname)
            all_declared['fnames'][fname]={"file":None,"aliases":vals}
    for file in sub_files:
        contents = read_from_file(file)
        kinds = decl_kinds(contents)
        types = roll_types(kinds)
        all_declared["kinds"].update(kinds)
        all_declared["scripts"][file] = {"kinds":kinds,"types":{},"fnames":[]}
        for key,values in types.items():
            if key not in all_declared["types"]:
                all_declared["types"][key] = []
            all_declared["types"][key] = update_list_value(all_declared["types"][key],values)    
        all_declared = fnames_in_kinds(all_declared,file)
    return all_declared

directory = "/var/www/modules/packages/abstract-utilities/src"
dirs,json_paths = get_files_and_dirs(directory,allowed_patterns='alias_map')
all_declared={}
for json_path in json_paths:
    all_declared = get_all_func_names(json_path,all_declared=all_declared)
    all_declared_scripts = all_declared.get("scripts")
    all_declared_fnames = all_declared.get('fnames')
    
    for sub_file,values in all_declared_scripts.items():
        
        
        for fname in all_declared_fnames:
            contents = read_from_file(sub_file)
      
            fname_values = all_declared_fnames.get(fname)
            aliases = fname_values.get('aliases')
            
            all_aliases = []
            lines = contents.split('\n')
            aliases = list(set([alias for alias in fname_values.get('aliases') if alias not in decl_kinds(contents)]))
            for i,line in enumerate(lines):
                if line.startswith(f'export function {fname}'):
                    string = '\n'.join(lines[i:])
                
                    
                    alias_funcs = create_alias_funcs(string,fname,aliases)
                    all_aliases+=alias_funcs
            if all_aliases:
                all_aliases = [contents]+all_aliases
                contents = '\n'.join(all_aliases)
                
                write_to_file(contents=contents,file_path=sub_file)
                




from .imports import *

def discover_pdfs(config: SiteConfig):
    """
    Yield one record per PDF directory.
    Expected layout:
        {slug}/{slug}.pdf
        {slug}/pages/0001/image.png, info.json, metadata.json, text.txt
        {slug}/meta/metadata.json
    """
    for pdf_path in find_files_by_ext(config.pdfs_dir, {".pdf"}):
        pdf_dir = os.path.dirname(pdf_path)
        slug = os.path.basename(pdf_dir)
        meta_path = os.path.join(pdf_dir, "meta", "metadata.json")
        html_path = os.path.join(pdf_dir, "index.html")

        metadata = safe_load_json(meta_path)
        title = metadata.get("title") or humanize(slug)
        description = clean_text(
            metadata.get("summary")
            or metadata.get("description")
            or "Read %s in image, text, or PDF view." % title
        )

        # discover pages
        pages_dir = os.path.join(pdf_dir, "pages")
        pages = []
        if os.path.isdir(pages_dir):
            for page_name in sorted(os.listdir(pages_dir)):
                page_dir = os.path.join(pages_dir, page_name)
                if not os.path.isdir(page_dir) or not page_name.isdigit():
                    continue
                page_num = int(page_name)
                page_image = first_existing_file(
                    os.path.join(page_dir, "image.webp"),
                    os.path.join(page_dir, "image.png"),
                    os.path.join(page_dir, "image.jpg"),
                    os.path.join(page_dir, "image.jpeg"),
                )
                page_thumb = first_existing_file(
                    os.path.join(page_dir, "image_627x1200.png"),
                    
                    os.path.join(page_dir, "resized_image.jpg"),
                    os.path.join(page_dir, "image.png"),
                    page_image,
                )
                page_info_path = os.path.join(page_dir, "info.json")
                page_meta_path = os.path.join(page_dir, "metadata.json")
                page_text_path = os.path.join(page_dir, "text.txt")
                page_html_path = os.path.join(page_dir, "index.html")

                page_info = safe_load_json(page_info_path)
                page_meta = safe_load_json(page_meta_path)

                page_keywords_raw = []
                kw_obj = page_info.get("keywords", {})
                if isinstance(kw_obj, dict):
                    page_keywords_raw = kw_obj.get("primary") or kw_obj.get("meta_keywords", "")
                    if isinstance(page_keywords_raw, str):
                        page_keywords_raw = [k.strip() for k in page_keywords_raw.split(",") if k.strip()]

                page_title = page_meta.get("title") or page_info.get("scope") or ""
                page_alt = page_meta.get("alt") or "%s page %s" % (slug, page_name)
                page_desc = page_meta.get("description") or clean_text(
                    safe_read_text(page_text_path) or page_alt
                )

                pages.append({
                    "page_number": page_num,
                    "page_name": page_name,
                    "dir_path": page_dir,
                    "image_path": page_image,
                    "thumb_path": page_thumb,
                    "html_path": page_html_path,
                    "info_path": page_info_path,
                    "metadata_path": page_meta_path,
                    "text_path": page_text_path,
                    "title": page_title,
                    "alt": page_alt,
                    "description": page_desc,
                    "keywords": normalize_keywords(metadata,page_keywords_raw),
                    "text": safe_read_text(page_text_path),
                    "info": page_info,
                    "metadata": page_meta,
                })

        # aggregate keywords from all pages
        all_kw = []
        for pg in pages:
            all_kw.extend(pg["keywords"])
        meta_kw = metadata.get("keywords") or []
        if isinstance(meta_kw, str):
            meta_kw = [k.strip() for k in meta_kw.split(",") if k.strip()]
        combined_kw = normalize_keywords(metadata,meta_kw + all_kw, limit=20)

        # thumbnail: first page thumb
        first_thumb = pages[0]["thumb_path"] if pages else ""

        yield make_record(
            kind="pdf",
            slug=slug,
            dir_path=pdf_dir,
            title=title,
            description=description,
            keywords=combined_kw,
            source_path=pdf_path,
            html_path=html_path,
            metadata_path=meta_path,
            thumbnail_path=first_thumb,
            metadata=metadata,
            pages=pages,
            page_count=len(pages),
        )


